
#pragma once

#include <initguid.h>
#include <dshow.h>

#include "strmif.h"
#include "usgfw.h"
#include "Usgfw2_h.h"
#include "afxwin.h"

//--------------------------------------------------------------------------

class ScanConverterCallback : public IUsgfwScanConverterPluginCB2
{
public:
	ScanConverterCallback();
	~ScanConverterCallback();
	//void SetUsgWnd(HWND wnd, int x, int y);
	void SetPlugin(IUsgfwScanConverterPlugin4* plugin);

private:
	IUsgfwScanConverterPlugin4* m_plugin;
	//HWND m_usg_wnd;
	//int m_usg_x;
	//int m_usg_y;

public:

// begin: IUnknown
	virtual HRESULT __stdcall QueryInterface( REFIID riid, void **ppObj);
	virtual ULONG STDMETHODCALLTYPE AddRef( void );
	virtual ULONG STDMETHODCALLTYPE Release( void );
	long m_nRefCount;
// end: IUnknown

// begin: IUsgfwScanConverterPluginCB2

	// method is called with input sample as parameter.
	// method is called just after sample is received into scan-converter
	STDMETHOD(InputSampleCB)(THIS_
		IMediaSample* pSample,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		);

	// method is called with input buffer as parameter.
	// method is called just after sample is received into scan-converter
	STDMETHOD(InputBufferCB)(THIS_
		PBYTE pBuffer,
		LONG nBufferSize,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with interim buffer as parameter.
	// method is called just after interim data is created from input data in scan-converter
	STDMETHOD(InterimBufferCB)(THIS_
		PBYTE pBuffer,
		LONG nBufferSize,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with output sample as parameter.
	// method is called just before output sample will be passed to downstream filter
	STDMETHOD(OutputSampleCB)(THIS_
		IMediaSample* pSample,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with output buffer as parameter.
	// method is called just before output data will be passed to downstream filter
	STDMETHOD(OutputBufferCB)(THIS_
		PBYTE pBuffer,
		LONG nBufferSize,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with input and output samples specified
	// method is called just before output sample will be passed to downstream filter
	STDMETHOD(SampleCB) (THIS_
		IMediaSample *pSampleIn,
		IMediaSample *pSampleOut,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with input and output buffers specified as parameters
	// method is called just before output sample will be passed to downstream filter
	STDMETHOD(BufferCB) (THIS_
		PBYTE pBufferIn,
		int nInBufferLen,
		PBYTE pBufferOut,
		int nOutBufferLen,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with pointers to input and intermediate buffers specified as parameters
	// method is called just after data is processed from input buffer into the intermediate buffer
	STDMETHOD(InInterimBufferCB) (THIS_
		PBYTE pBufferIn,
		int nInBufferLen,
		PBYTE pBufferInterim,
		int nInterimBufferLen,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with pointers to input sample and intermediate buffer specified as parameters
	// method is called just after data is processed from input sample into the intermediate buffer
	STDMETHOD(InInterimSampleCB) (THIS_
		IMediaSample *pSampleIn,
		PBYTE pBufferInterim,
		int nInterimBufferLen,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with pointers to intermediate buffer and output buffer specified as parameters
	// method is called just before output data buffer will be passed to downstream filter
	STDMETHOD(InterimOutBufferCB) (THIS_
		PBYTE pBufferInterim,
		int nInterimBufferLen,
		PBYTE pBufferOut,
		int nOutBufferLen,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with pointers to intermediate buffer and output sample specified as parameters
	// method is called just before output sample will be passed to downstream filter
	STDMETHOD(InterimOutSampleCB) (THIS_
		PBYTE pBufferInterim,
		int nInterimBufferLen,
		IMediaSample *pSampleOut,
		LONG Param1,
		LONG Param2,
		LONG Param3,
		LONG Param4
		) { return S_OK; }

	// method is called with new media type specified as parameter
	// method is called before callbacks with new format data be invoked
	STDMETHOD(MediaTypeCB) (THIS_
		AM_MEDIA_TYPE *pMediaType,
		BOOL bOutputPin
		) { return S_OK; }

	// method is called when some scan-conversion parameter was changed
	STDMETHOD(ParameterCB)(THIS_
		void
		) { return S_OK; }

// end: IUsgfwScanConverterPluginCB2

}; // class ScanConverterCallback

//--------------------------------------------------------------------------

class CsampleDlg : public CDialog
{
public:
	CsampleDlg(CWnd* pParent = NULL);	// standard constructor
	~CsampleDlg();

// Dialog Data
	enum { IDD = IDD_SAMPLE_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

protected:
	HICON m_hIcon;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();

	DECLARE_MESSAGE_MAP()

private:
	IUsgfw2* m_usgfw2;
	IUsgDataView* m_data_view;
	//IProbe* m_probe;
	IUsgMixerControl* m_mixer_control;
	IUsgFileStorage* m_storage;
	IUsgCineStream* m_cine;
	IUsgfwScanConverterPlugin4* m_plugin;
	ScanConverterCallback* m_scan_converter_callback;

	void CreateUsgControl( IUsgDataView* data_view, const IID& type_id, ULONG scan_mode, ULONG stream_id, void** ctrl );
	void CreateUsgControls();
	void ReleaseUsgControls();
	void OpenFile();
	void ConvertFile();

public:
	afx_msg void OnClose();
public:
	afx_msg BOOL OnQueryEndSession();
public:
	afx_msg void OnEndSession(BOOL bEnding);

public:
	CEdit m_file_ctrl;
public:
	afx_msg void OnBnClickedBrowseButton();
public:
	afx_msg void OnBnClickedOpenButton();
public:
	afx_msg void OnBnClickedConvertButton();
public:
	CStatic m_status_ctrl;
public:
	CButton m_save_frame_format_info_checkbox;
public:
	afx_msg void OnBnClickedSaveFrameFormatInfoCheckbox();
public:
	CButton m_save_scan_lines_checkbox;
public:
	afx_msg void OnBnClickedSaveScanLinesCheckbox();
public:
	afx_msg void OnBnClickedStopConversionButton();
}; // class CsampleDlg

//--------------------------------------------------------------------------

#define SAFE_RELEASE(x) { if (x) x->Release(); x = NULL; }

//--------------------------------------------------------------------------
